@@ -6,6 +6,8 @@
 
 #![cfg_attr(not(feature = "std"), no_std)]
 
+use core::convert::TryInto;
+
 use utils_results::*;
 
 pub extern crate derivation_path;
@@ -17,7 +19,7 @@ pub use ed25519_dalek::{PublicKey, SecretKey};
 use hmac::{Hmac, Mac, NewMac};
 use sha2::Sha512;
 
-const ED25519_BIP32_NAME: &str = "ed25519 seed";
+type HmacSha512 = Hmac<Sha512>;
 
 err! {
     Hmac => "hmac error"
@@ -26,22 +28,71 @@ err! {
     ExpectedHardenedIndex => "expected hardened child index:"
 }
 
-/// An expanded secret key with chain code and meta data
+const ED25519_BIP32_NAME: &str = "ed25519 seed";
+
+pub const EXTENDED_KEY_LENGTH: usize = 69;
+
 #[derive(Debug)]
 pub struct ExtendedSecretKey {
-    /// How many derivations this key is from the root (0 for root)
-    pub depth: u8,
-    /// Child index of the key used to derive from parent (`Normal(0)` for root)
-    pub child_index: ChildIndex,
-    /// Secret Key
-    pub secret_key: SecretKey,
-    /// Chain code
+    /// (0 for root)
+    pub depth: u8, // 1 byte
+    /// used to derive from parent (`Normal(0)` for root)
+    pub child_index: ChildIndex, // 4 bytes
+    pub secret_key: SecretKey, // 32 bytes
     pub chain_code: [u8; 32],
 }
 
-type HmacSha512 = Hmac<Sha512>;
+impl PartialEq for ExtendedSecretKey {
+    fn eq(&self, other: &Self) -> bool {
+        (self.depth == other.depth)
+            && (self.child_index == other.child_index)
+            && (self.secret_key.as_bytes() == other.secret_key.as_bytes())
+            && (self.chain_code == other.chain_code)
+    }
+}
+impl Eq for ExtendedSecretKey {}
 
 impl ExtendedSecretKey {
+    #[inline(always)]
+    fn convert_to_bytes(xprv: &Self) -> [u8; EXTENDED_KEY_LENGTH] {
+        let depth = [xprv.depth; 1];
+        let child_index = xprv.child_index.to_bits().to_be_bytes();
+        let secret_key = xprv.secret_key.to_bytes();
+        let mut out = [0u8; EXTENDED_KEY_LENGTH];
+        for (out_i, &in_i) in out.iter_mut().zip(
+            depth.iter().chain(
+                child_index
+                    .iter()
+                    .chain(secret_key.iter().chain(xprv.chain_code.iter())),
+            ),
+        ) {
+            *out_i = in_i;
+        }
+        out
+    }
+    pub fn to_bytes(&self) -> [u8; EXTENDED_KEY_LENGTH] {
+        Self::convert_to_bytes(self)
+    }
+    pub fn into_bytes(self) -> [u8; EXTENDED_KEY_LENGTH] {
+        Self::convert_to_bytes(&self)
+    }
+    pub fn from_bytes(bytes: &[u8; EXTENDED_KEY_LENGTH]) -> Result<Self> {
+        let depth = bytes[0];
+        let child_index = ChildIndex::from_bits(u32::from_be_bytes(errcast!(
+            (&bytes[1..5]).try_into(),
+            err::Parse
+        )));
+        let secret_key = errcast!(SecretKey::from_bytes(&bytes[5..37]), err::Parse);
+        let mut chain_code = [0u8; 32];
+        chain_code.copy_from_slice(&bytes[37..69]);
+        Ok(Self {
+            depth,
+            child_index,
+            secret_key,
+            chain_code,
+        })
+    }
+
     /// Create a new extended secret key from a seed
     pub fn from_seed(seed: &[u8]) -> Result<Self> {
         let mut mac = errcast!(
@@ -132,6 +183,14 @@ mod tests {
         ExtendedSecretKey::from_seed(&hex_str(seed)).unwrap()
     }
 
+    #[test]
+    fn bytes_conversion() {
+        let xprv = root("000102030405060708090a0b0c0d0e0f");
+        let bytes = xprv.to_bytes();
+        let xprv2 = ExtendedSecretKey::from_bytes(&bytes).unwrap();
+        assert_eq!(xprv, xprv2);
+    }
+
     #[test]
     fn derivation_path() {
         let vector1_path: DerivationPath = "m/0'/1'/2'/2'/1000000000'".parse().unwrap();
